﻿using System;
using System.Diagnostics;
using System.IO;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using Microsoft.Sample.Controls;

namespace Minemapper
{
	public class Tile : IVirtualChild
	{
		public static Uri EmptyImageUri;

		public double X { get; set; }
		public double Z { get; set; }

		public double Left { get; set; }
		public double Top { get; set; }

		public int WorldHeight { get; set; }
		public int CurrentImageWorldHeight { get; set; }

		public FileInfo FileInfo { get; set; }
		public TransparentImage Image { get; set; }

		public Action<Tile> GenerateTileImageCallback { get; set; }

		public bool IsBlank { get; set; }

		#region IVirtualChild Members

		public Rect Bounds { get; set; }

		public UIElement Visual
		{
			get { return this.Image; }
		}

		public UIElement CreateVisual()
		{
			try
			{
				this.Image = new TransparentImage();

				this.Image.Tag = this;

				this.FileInfo.Refresh();

				if (this.FileInfo.Exists)
				{
					this.IsBlank = this.FileInfo.Length <= 1163;

					if (this.IsBlank)
					{
						this.Image.Source = null;
						this.Image.Visibility = Visibility.Hidden;
					}
					else
					{
						double top = (this.Z * 16) + (this.X * -16);

						top += (128 - this.WorldHeight) * 2;

						this.Top = top;

						RenderOptions.SetBitmapScalingMode(this.Image, BitmapScalingMode.NearestNeighbor);
						RenderOptions.SetEdgeMode(this.Image, EdgeMode.Aliased);

						BitmapImage bitmapImage = new BitmapImage();
						bitmapImage.BeginInit();
						bitmapImage.CacheOption = BitmapCacheOption.OnDemand;
						bitmapImage.UriSource = new Uri(this.FileInfo.FullName);
						bitmapImage.EndInit();

						this.Image.Source = bitmapImage;

						if (this.Image.Visibility != Visibility.Visible)
							this.Image.Visibility = Visibility.Visible;
					}
				}
				else
				{
					RenderOptions.SetBitmapScalingMode(this.Image, BitmapScalingMode.HighQuality);
					RenderOptions.SetEdgeMode(this.Image, EdgeMode.Unspecified);

					if (EmptyImageUri == null)
					{
						string path = Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location);
						path = Path.Combine(path, "empty.png");
						EmptyImageUri = new Uri(path);
					}

					this.Image.Source = new BitmapImage(EmptyImageUri);
					this.CurrentImageWorldHeight = this.WorldHeight;
					if (this.GenerateTileImageCallback != null)
						this.GenerateTileImageCallback(this);
				}

				this.Image.Tag = this;

				this.Image.SetValue(Canvas.LeftProperty, this.Left);
				this.Image.SetValue(Canvas.TopProperty, this.Top);

				int zindex = (int)-this.X + (int)this.Z;

				this.Image.SetValue(Canvas.ZIndexProperty, zindex);
			}
			catch (Exception ex)
			{
				Debug.WriteLine(ex);
			}

			return this.Image;
		}

		public void DisposeVisual()
		{
			try
			{
				this.Image.Source = null;
				this.Image = null;
			}
			catch (Exception ex)
			{
				Debug.WriteLine(ex);
			}
		}

		#endregion
	}

	public delegate void TileEventHandler(object sender, EventArgs<Tile> e);

	public class EventArgs<T> : EventArgs
	{
		public T Item { get; private set; }

		public EventArgs(T item)
		{
			this.Item = item;
		}
	}
}
